[comment {-*- tcl -*- doctools manpage}]
[vset libname dbif]
[vset cmdname dbif]
[vset version 2.2]
[manpage_begin [vset libname] n [vset version]]
[copyright {2008-2017 Schelte Bron}]
[moddesc {Tcl D-Bus Interface}]
[titledesc {Application layer around the Tcl D-Bus library}]
[require Tcl 8.5]
[require dbus "3.0"]
[require [vset libname] [vset version]]
[description]

The [package [vset libname]] package provides a higher level wrapper around
the low-level D-Bus commands provided by the dbus package. The package also
provides an implementation of a number of standard D-Bus interfaces.
See [sectref "STANDARD INTERFACES"] for more information.

[para]

Access to all functions of the [package [vset libname]] package from within
a Tcl program is done using the [cmd [vset cmdname]] command. The command
supports several subcommands that determine what action is carried out.

[list_begin definitions]
[call [cmd [vset cmdname]] [method close] \
  [opt "[option -bus] [arg bustype]"]]

Close the dbus connection, cleaning up all associated signals, methods, and
properties.

[call [cmd [vset cmdname]] [method connect] \
  [opt "[option -bus] [arg bustype]"] [opt [option -noqueue]] \
  [opt [option -replace]] [opt [option -yield]] [opt "[arg name] ..."]]

Connect to a message bus and optionally request the D-Bus server to assign
one or more names to the current application.

[para]

The [option -yield] option specifies that the application will release the
requested name when some other application requests the same name and has
indicated that it wants to take over ownership of the name. The application
will be informed by a [emph NameLost] signal when it loses ownership of the
name.

[para]

The [option -replace] option indicates that the application wants to take
over the ownership of the name from the application that is currently the
primary owner, if any. This request will only be honoured if the current
owner has indicated that it will release the name on request. See also the
[option -yield] option.

[para]

If the requested name is currently in use and the -replace option has not
been specified, or the [option -replace] option was specified but the
current owner is unwilling to give up its ownership, the name request will
normally be queued. Then when the name is released by the current owner it
is assigned to the next requester in the queue and a signal is sent to
inform that requester that it is now the primary owner of the name. The
[option -noqueue] option may be specified to indicate that the name request
should not be queued.

[para]

The command returns a list of names that have successfully been acquired.
If the dbus connection handle is needed, it can be obtained from the -bus
return option.

[para]

The following code can be used to allow a new instance of a program to 
replace the current one. This can be useful during program development:
[example_begin]
dbif connect -yield -replace $dbusname
dbif listen -interface [lb]dbus info service[rb] \
  [lb]dbus info path[rb] NameLost name {if {$name eq $::dbusname} exit}
[example_end]

[call [cmd [vset cmdname]] [method default] \
  [opt "[option -bus] [arg bustype]"] \
  [opt "[option -interface] [arg interface]"]]

Generally an application will perform several [cmd [vset cmdname]] commands
related to the same message bus and interface. To avoid having to pass the
same values for the [option -bus] and [option -interface] options with all
those commands, their defaults can be setup with the [cmd [vset cmdname]]
[method default] subcommand.

[para]

An interface name must consist of at least two elements separated by
a period ('.') character. Each element may only contain the characters
"[lb]A-Z[rb][lb]a-z[rb][lb]0-9[rb]_" and must not begin with a digit.

[para]

The initial value for [option -bus] is session. The initial value for
[option -interface] is taken from the first name requested for the
application in a [cmd [vset cmdname]] [method connect] command. If no
name was ever requested with the connect subcommand, it defaults to
"com.tclcode.default".

[call [cmd [vset cmdname]] [method delete] \
  [opt "[option -bus] [arg bustype]"] \
  [opt "[option -interface] [arg interface]"] \
  [opt "[option -single]"] [arg path]]

While there currently is no way to remove individual signals, methods, or
properties from the published interface, this subcommand allows the removal
of a complete node. Unless the [option -single] option is specified, the
command will also recursively delete nodes on all underlying object paths.

[call [cmd [vset cmdname]] [method error] [arg messageID] \
  [arg errormessage] [opt [arg errorname]]]

Send a D-Bus error message in response to a D-Bus method call. If the
[arg errorname] argument is not specified, it defaults to
"org.freedesktop.DBus.Error.Failed".

[call [cmd [vset cmdname]] [method generate] \
  [arg signalID] [opt "[arg arg] ..."]]

Generate a signal as defined by a previous [cmd [vset cmdname]] 
[method signal] command. If a body was specified with the signal definition,
the provided arguments must match the args definition for the body. Otherwise
they must match the arglist specified during the definition of the signal.

[call [cmd [vset cmdname]] [method get] [arg messageID] [arg name]]

Access additional information about a D-Bus message. Recognized names are
[arg bus], [arg member], [arg interface], [arg path], [arg sender],
[arg destination], [arg messagetype], [arg signature], [arg serial],
[arg replyserial], [arg noreply], [arg autostart], and [arg errorname].

[call [cmd [vset cmdname]] [method listen] \
  [opt "[option -bus] [arg bustype]"] \
  [opt "[option -interface] [arg interface]"] \
  [arg path] [arg name] [opt [arg arglist]] [arg body]]

Start listening for the specified signal and execute [arg body] when such a
signal appears on the D-Bus. If [arg body] is specified as an empty string,
the signal handler with a matching argument signature is deleted.

[para]

The code in [arg body] will be executed in the namespace the
[cmd [vset cmdname]] [method listen] command was issued from.

The [arg arglist] argument follows the special rules for [vset cmdname]
argument lists. See [sectref "ARGUMENT LISTS"] below for more information.

[call [cmd [vset cmdname]] [method method] \
  [opt "[option -attributes] [arg attributes]"] \
  [opt "[option -bus] [arg bustype]"] \
  [opt "[option -interface] [arg interface]"] \
  [arg path] [arg name] \
  [opt "[arg inputargs] [opt [arg outputargs]]"] [arg body]]

Define a method that may be accessed through the D-Bus and execute [arg body]
when the method is invoked. 
In addition to valid dbus paths, an empty string may be specified for the
[arg path] argument. This makes the method available on all paths.
The [arg inputargs] argument specifies which arguments must be provided by
the caller. The [arg outputargs] argument indicates the type of result the
method returns.

[para]

Attributes may be specified via the [option -attributes] option to provide
hints to users of your API.
See [sectref ATTRIBUTES] below for more information.

[para]

The return value resulting from executing the body will normally be returned
to the caller in a D-Bus return message. If an uncaught error occurs or the
result of body doesn't match [arg outputargs], an error message will be
returned to the caller instead.

[para]

The body code recognizes an additional [option -async] option for the Tcl
[cmd return] command. When that option is specified with a true boolean
value ([const true], [const yes], [const 1]), the return value from the body
will not automatically be returned to the caller. A response message should
then be generated using the [cmd [vset cmdname]] [method return] or
[cmd [vset cmdname]] [method error] subcommands.

[para]

An additional variable [var msgid] will be passed to the method body. This
variable contains a messageID that may be used in combination with the
[method get], [method return], or [method error] subcommands. The messageID
remains valid for a period of time (default 25 seconds), or until a response
has been returned to the caller, whichever happens first.

[para]

The code in [arg body] will be executed in the namespace the
[cmd [vset cmdname]] [method method] command was issued from.

The [arg inputargs] and [arg outputargs] arguments follow the special rules
for [vset cmdname] argument lists.
See [sectref "ARGUMENT LISTS"] below for more information.

[call [cmd [vset cmdname]] [method pave] \
  [opt "[option -bus] [arg bustype]"] \
  [opt "[option -interface] [arg interface]"] [arg path]]

Create a path. Normally a path is automatically created when a property,
method, or signal is defined on that path. This command is intended for paths
that only have universal methods and/or signals (i.e. methods and/or signals
that are defined with an empty string as their path).

[call [cmd [vset cmdname]] [method property] \
  [opt "[option -access] [arg mode]"] \
  [opt "[option -attributes] [arg attributes]"] \
  [opt "[option -bus] [arg bustype]"] \
  [opt "[option -interface] [arg interface]"] \
  [arg path] [arg name]?:[arg signature]? [arg variable] \
  [opt [arg body]]]

Define a property that may be accessed through the D-Bus using methods defined 
by the org.freedesktop.DBus.Properties standard interface. The [arg variable]
argument defines the global variable holding the value of the property.

The [arg signature] of a property must be a single complete type.
[para]

The [option -access] option specifies whether the property can be viewed
and/or modified through the D-Bus. Valid access modes are [const read],
[const write], and [const readwrite]. If no access mode is specified, it
defaults to readwrite.

[para]

Attributes may be specified via the [option -attributes] option to provide
hints to users of your API.
See [sectref ATTRIBUTES] below for more information.

[para]

The code in the optional [arg body] argument will be executed when the
property is modified through the D-Bus. During the execution of [arg body]
the global variable will still have its original value, if any. The new
value for the property is passed to the script as an argument with the same
name as the property. If execution of [arg body] results in an error, the
global variable will not be modified. This allows restrictions to be imposed
on the value for the property.

[para]

The code in [arg body] will be executed in the namespace the
[cmd [vset cmdname]] [method property] command was issued from.

[para]

Generating the property value only when needed can be implemented by putting
a read trace on the global variable. Example:
[example_begin]
dbif property -attributes {Property.EmitsChangedSignal false} / clock sec
trace add variable sec read {apply {args {set ::sec [lb]clock seconds[rb]}}}
[example_end]

In this example the Property.EmitsChangedSignal attribute is used to
prevent the [emph PropertiesChanged] signal being generated, which would
involve a second read of the variable.

[call [cmd [vset cmdname]] [method return] [arg messageID] \
  [arg returnvalue]]

Send a D-Bus return message in response to a D-Bus method call. The provided
[arg returnvalue] must match the signature specified earlier in the
[cmd [vset cmdname]] [method method] command for the method.

[call [cmd [vset cmdname]] [method signal] \
  [opt "[option -attributes] [arg attributes]"] \
  [opt "[option -bus] [arg bustype]"] \
  [opt "[option -id] [arg signalID]"] \
  [opt "[option -interface] [arg interface]"] \
  [arg path] [arg name] \
  [opt "[arg arglist] [opt "[arg args] [arg body]"]"]]

Define a signal that the application may emit using the [cmd [vset cmdname]]
[method generate] subcommand. Signals are referred to by their SignalID.
If [option -id] is specified, it is used as the SignalID. Otherwise a new
unique identifier is generated. Specifying an existing SignalID replaces
the previously defined signal.

[para]

Attributes may be specified via the [option -attributes] option to provide
hints to users of your API.
See [sectref ATTRIBUTES] below for more information.

[para]

The command returns the SignalID of the newly created signal.

[para]

If the optional [arg args] and [arg body] arguments are specified, body
will be executed when the signal is transmitted on the D-Bus as a result
of the [cmd [vset cmdname]] [method generate] subcommand. It is the
responsibility of the body code to produce a return value that matches
the specified arglist.

[para]

The code in [arg body] will be executed in the namespace the
[cmd [vset cmdname]] [method signal] command was issued from.

If any uncaught error happens during the execution of the body code, the
[cmd [vset cmdname]] [method generate] command will also throw an error with
the same error message. 

When the body code comes to the conclusion that the signal doesn't need to
be sent after all, it may abort the operation by returning using
[lb]return -code return[rb].

The [arg arglist] argument follows the special rules for [vset cmdname]
argument lists. See [sectref "ARGUMENT LISTS"] below for more information.

[para]

In addition to valid dbus paths, an empty string may be specified for the
[arg path] argument. This makes the signal available on all paths. In this
case a body must be provided and the body code must provide a path in the
[option -path] option to the [cmd return] command.
For example: The following helper proc could be used to allow providing a
path to the [cmd [vset cmdname]] [method generate] command in front of the
signal arguments:
[example_begin]
proc stdsignal {path args} {
    # Single argument signal bodies are not expected to produce a list
    if {[lb]llength $args[rb] == 1} {set args [lb]lindex $args 0[rb]}
    return -path $path $args
}
[example_end]

[list_end]

[section "BUS TYPES"]
The [option -bus] option of the various subcommands takes a [arg bustype]
value that can take several forms:
[list_begin itemized]
[item]One of the well-known bus names: 'session', 'system', or 'startup'.
[item]A bus address, consisting of a transport name followed by a colon,
and then an optional, comma-separated list of keys and values in the form
key=value.
[item]A handle as returned by the [cmd dbus] [method connect] subcommand.
[list_end]

[section "VALID NAMES"]

The [vset libname] package enforces some limitations on names used with the
[cmd [vset cmdname]] subcommands. All names must only use the characters
"[lb]A-Z[rb][lb]a-z[rb][lb]0-9[rb]_". This limitation applies to method
names, property names, signal names, and argument names. Out of this group,
only argument names may begin with a digit.

[para]

Interface names and error names must consist of at least two elements
separated by a period ('.') character. Each element must only contain the
characters "[lb]A-Z[rb][lb]a-z[rb][lb]0-9[rb]_" and must not begin with a
digit.

[para]

D-Bus names for applications must follow the same rules as interface names,
except that also dash ('-') characters are allowed. Unique D-Bus names begin
with a colon (':'). The elements of unique D-Bus names are allowed to begin
with a digit.

[para]

Paths must start with a slash ('/') and must consist of elements separated
by slash characters. Each element must only contain the characters
"[lb]A-Z[rb][lb]a-z[rb][lb]0-9[rb]_". Empty elements are not allowed.

[section "ARGUMENT LISTS"]
Due to the fact that the D-Bus specification works with typed arguments, a
slightly modified method for specifying argument lists has been adopted for
the [vset libname] package. The normal Tcl argument list as used with the
[cmd proc] and [cmd apply] commands may still be used. In that case all
arguments will be of the string type. To specify a different type, the
variable name must be followed by a colon and the D-Bus signature
specification for that argument. Example: to define the count argument as
a 32-bit integer, it should be specified as count:i.

[para]

The following argument types are available:
[list_begin definitions]
[def s] A UTF-8 encoded, nul-terminated Unicode string.
[def b] A boolean, FALSE (0), or TRUE (1).
[def y] A byte (8-bit unsigned integer).
[def n] A 16-bit signed integer.
[def q] A 16-bit unsigned integer.
[def i] A 32-bit signed integer.
[def u] A 32-bit unsigned integer.
[def x] A 64-bit signed integer.
[def t] A 64-bit unsigned integer.
[def d] An 8-byte double in IEEE 754 format.
[def g] A type signature.
[def o] An object path.
[def a#] A D-Bus array type, which is similar to a Tcl list. The # specifies
the type of the array elements. This can be any type, including another
array, a struct or a dict entry.
[def v] A D-Bus variant type. Specifying this type will cause the code to
automatically determine the type of the provided value (by looking at the
internal representation).
[def (...)] A struct. The string inside the parentheses defines the types of
the arguments within the struct, which may consist of a combination of any
of the existing types.
[def {{##}}] A dict entry. Dict entries may only occur as array elements. The
first # specifies the type of the dict key. This must be a basic type
(one of 'sbynqiuxtdgo'). The second # specifies the type of the dict value.
This can again be any existing type.
[list_end]

[para]

Argument lists may contain optional arguments. The use of optional arguments
will result in multiple prototypes being reported for the object when
introspected. The special meaning of the [const args] argument does not
translate well in the D-Bus concept. For that reason using [arg args] as the
last argument of an argument list should be avoided.

[section "STANDARD INTERFACES"]

A number of standard interfaces have been defined in the D-Bus specification
that may be useful across various D-Bus applications.

[subsection org.freedesktop.DBus.Peer]
[list_begin definitions]
[def [emph org.freedesktop.DBus.Peer.Ping]]
Returns an empty response.

[def [emph org.freedesktop.DBus.Peer.GetMachineId]]
Returns a hex-encoded UUID representing the identity of the machine the
application is running on.
[list_end]

[subsection org.freedesktop.DBus.Introspectable]
[list_begin definitions]
[def [emph org.freedesktop.DBus.Introspectable.Introspect]]
Returns an XML description of the D-Bus structure, including its interfaces
(with signals and methods), objects below it in the object path tree, and
its properties.
[list_end]

[subsection org.freedesktop.DBus.Properties]
[list_begin definitions]
[def [emph org.freedesktop.DBus.Properties.Get]]
Returns the value of the specified property. Only valid for properties with
read or readwrite access.
[def [emph org.freedesktop.DBus.Properties.Set]]
Changes the value of the specified property. Only valid for properties with
write or readwrite access.
[def [emph org.freedesktop.DBus.Properties.GetAll]]
Returns a dict of all properties with read or readwrite access.
[def [emph org.freedesktop.DBus.Properties.PropertiesChanged]]
This signal is emitted when one or more properties change.
The behavior for individual properties may be influenced by their
Property.EmitsChangedSignal attribute. See [sectref ATTRIBUTES] below.

[para]

All applicable property changes are collected and reported via a single
[emph PropertiesChanged] signal per path/interface/bus combination when the
application enters the idle loop. The signal may also be generated on demand
via the command: 
[list_begin definitions]
[list_begin definitions]
[def "[cmd [vset cmdname]] [method generate] PropertiesChanged\
[arg path] [opt [arg interface]] [opt [arg bus]]"]
[list_end]
[list_end]

[para]

The functionality for automatically populating this signal is implemented
via variable traces. If this feature is not needed, you can avoid the
associated overhead by redefining the PropertiesChanged signal id to your
own version.
To completely get rid of the signal, you can use the following code snippet
before defining any other part of your dbus interface:
[example_begin]
package require dbif
dbif signal -id PropertiesChanged / foobar
dbif delete /
[example_end]

[list_end]

[section "ATTRIBUTES"]
Attributes may be specified as a list of key/value pairs for methods,
signals, and properties. These attributes are reported via annotations in
the XML description obtained via an [emph Introspect] method call.
Annotations may be used to provide hints to users of your API.

[para]

Some well-know attributes are (default, if any, shown in italics):
[list_begin definitions]
[def Description]
Provide a short 1-line description of the method, signal or property.
[def Deprecated]
Indicate that this method is deprecated (true, [emph false]).
[def Method.NoReply]
This method may not produce a reply (true, [emph false]). For example if you
provide a method to exit your application.
[def Method.Error]
This method may throw the indicated Exception in addition to the standard
ones.
[def Property.EmitsChangedSignal]
Indicates whether a change to the property is reported via the 
[emph PropertiesChanged] signal ([emph true], false, invalidates, const).
[para]
The value of this attribute, if specified, is also used internally to
influence the automatic generation of the [emph PropertiesChanged] signal.
[list_begin definitions]
[def true]
The signal is emitted with the value included. This is the default.
[def false]
The signal is not automatically emitted on a change.
Parties interested in the property should obtain it every time they need it.
The application code may still emit a [emph PropertiesChanged] signal
whenever desired.
This may be used for properties that are implemented with a read trace on
the global variable.
[def invalidates]
The signal is emitted but the value is not included in the signal. This may
be useful for properties that change much more frequently than they are
expected to be queried, and/or have large values.
[def const]
The property never changes its value during the lifetime of the object it
belongs to, and hence the signal is never emitted for it. 
[list_end]
[list_end]

[manpage_end]
